/* Name: main.c
 * Project: USBaspLoader
 * Author: Christian Starkjohann
 * Creation Date: 2007-12-08
 * Tabsize: 4
 * Copyright: (c) 2007 by OBJECTIVE DEVELOPMENT Software GmbH
 * License: GNU GPL v2 (see License.txt)
 * This Revision: $Id: main.c 730 2009-03-20 09:05:11Z cs $
 */

#include <string.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include <avr/wdt.h>
#include <util/delay.h>
#include "usbconfig.h"
#include "usb162.h"
#include "cdc162.h"

#include <avr/boot.h>

#define bit_set(port,bit)	{ port |= (1<<(bit));}
#define bit_clear(port,bit)	{ port &= ~(1<<(bit));}
#define min(a,b)		((a)<(b) ? (a):(b))

#define nop()	__asm__ volatile ("nop" )
#define	wdr()	__asm__ volatile("wdr" )

#ifndef ASP_LOG
#define asp_debug(ch)
#else
#define ASP_LOG_SIZE	0x80

static uint8_t asp_log[ASP_LOG_SIZE];
static uint8_t asp_log_idx;

void __asp_debug(uint8_t ch) {
	if (asp_log_idx < ASP_LOG_SIZE) {
		asp_log[asp_log_idx++] = ch;

	}
}
#define asp_debug(ch)	__asp_debug(ch)
#endif

static void leaveBootloader() __attribute__((__noreturn__));

#include "bootloaderconfig.h"


#ifdef NO_INTERRPT
#define __cli()
#define __sei()
#else
#define __cli()	__asm__ volatile ("cli" )
#define __sei()	__asm__ volatile ("sei" )
#endif

#if USE_DFU_CALL
extern void dfu_page_write(uint16_t addr);
extern void dfu_page_erase(uint16_t addr);
extern void dfu_page_fill(uint16_t addr,uint16_t data);
#endif

#if BOOTLOADER_EMU
#define EEPROM_SAVE_BASE	(E2END + 1 -0x10)
#define EEPROM_SAVE_SIZE	(4)
#else
#define EEPROM_SAVE_BASE	(E2END + 1)
#define EEPROM_SAVE_SIZE	(0)
#endif

#ifndef APP_FLASHEND
#define APP_FLASHEND	(BOOTLOADER_ADDRESS -1)
#endif

/* ------------------------------------------------------------------------ */

/* Request constants used by USBasp */
#define USBASP_FUNC_CONNECT         1
#define USBASP_FUNC_DISCONNECT      2
#define USBASP_FUNC_TRANSMIT        3
#define USBASP_FUNC_READFLASH       4
#define USBASP_FUNC_ENABLEPROG      5
#define USBASP_FUNC_WRITEFLASH      6
#define USBASP_FUNC_READEEPROM      7
#define USBASP_FUNC_WRITEEEPROM     8
#define USBASP_FUNC_SETLONGADDRESS  9
#define USBASP_FUNC_SETISPSCK       10

#define ISPCMD_CHIP_ERASE	    0xac
#define ISPCMD_READ_SIGNATURE	    0x30
#define ISPCMD_READ_EEPROM	    0xa0
#define ISPCMD_WRITE_EEPROM	    0xc0

#define ISPCMD_READ_FLASH_L	    0x20
#define ISPCMD_READ_FLASH_H	    0x28

#define ISPCMD_READ_FUSE_L	    0x50	/* lower/ext */
#define ISPCMD_READ_FUSE_H	    0x58	/* upper/lockbit */

/* Block mode data size */
#define USBASP_READBLOCKSIZE   200
#define USBASP_WRITEBLOCKSIZE  200

/* ------------------------------------------------------------------------ */

/* defaults if not in config file: */
#ifndef HAVE_EEPROM_PAGED_ACCESS
#   define HAVE_EEPROM_PAGED_ACCESS 0
#endif
#ifndef HAVE_EEPROM_BYTE_ACCESS
#   define HAVE_EEPROM_BYTE_ACCESS  0
#endif
#ifndef BOOTLOADER_CAN_EXIT
#   define  BOOTLOADER_CAN_EXIT     0
#endif

/* device compatibility: */
#ifndef GICR    /* ATMega*8 don't have GICR, use MCUCR instead */
#   define GICR     MCUCR
#endif

/* ------------------------------------------------------------------------ */
#if (FLASHEND > USHRT_MAX)
#   define CURRENT_ADDRESS  currentAddress.l
#   define addr_t           uint32_t
#else
#   define CURRENT_ADDRESS  currentAddress.w[0]
#   define addr_t           uint16_t
#endif

typedef union longConverter{
    addr_t  l;
    uint16_t  w[sizeof(addr_t)/2];
    uint8_t   b[sizeof(addr_t)];
}longConverter_t;

#if BOOTLOADER_CAN_EXIT
static uint8_t            requestBootLoaderExit;
#endif
static longConverter_t  currentAddress; /* in bytes */
static uint8_t            bytesRemaining;
static uint8_t            isLastPage;
#if HAVE_EEPROM_PAGED_ACCESS
static uint8_t            currentRequest;
#endif

#if !HAVE_SIGNATURE_ACCESS
static uint8_t  signatureBytes[4] PROGMEM = {
    SIGNATURE_0, SIGNATURE_1, SIGNATURE_2, 0
};
#endif

/* ------------------------------------------------------------------------ */


static void leaveBootloader()
{
    void (*nullVector)(void) __attribute__((__noreturn__)) = NULL;
    //DBG1(0x01, 0, 0);
    bootLoaderExit();
    __cli();
#if !BOOTLOADER_EMU
    GICR = (1 << IVCE);     /* enable change of interrupt vectors */
    GICR = (0 << IVSEL);    /* move interrupts to application flash section */
/* We must go through a global function pointer variable instead of writing
 *  ((void (*)(void))0)();
 * because the compiler optimizes a constant 0 to "rcall 0" which is not
 * handled correctly by the assembler.
 */
#else
    if (EEPROM_SAVE_SIZE)
       nullVector = (void *)(eeprom_read_word((void *)EEPROM_SAVE_BASE + 2)<<1);
#endif
    nullVector();
}

/* ------------------------------------------------------------------------ */
#if HAVE_FUSE_ACCESS
static inline uint8_t read_fuse(uint16_t addr) {
	SPMCSR |= (1<<SPMEN) | (1<<BLBSET);
	return __LPM(addr);
}
#endif

#if HAVE_SIGNATURE_ACCESS
static inline uint8_t read_signature(uint16_t addr) {
	SPMCSR |= (1<<SPMEN) | (1<<SIGRD);
	return  __LPM(addr);
}
#endif

#if HAVE_CHIP_ERASE && EEPROM_SAVE_SIZE
static inline void fill_vector() {
	uint16_t addr = 0;
	uint16_t data;
	while (addr < SPM_PAGESIZE) {
		if (addr < 4) {
			data = __LPM_word(addr + BOOTLOADER_ADDRESS);
		} else {
			data = 0xffff;
		}
#if USE_DFU_CALL
		__cli();
		dfu_page_fill(addr, data);
		__sei();
#else
		__cli();
		boot_page_fill(addr, data);
		__sei();
#endif
		addr += 2;
	}
#if USE_DFU_CALL
	__cli();
	dfu_page_write((uint16_t)0);
	__sei();
#else
	__cli();
	boot_page_write((uint16_t)0);
	__sei();
	boot_spm_busy_wait();
	__cli();
	boot_rww_enable();
	__sei();
#endif
}
#endif

uint8_t asp_setup(uint8_t *in, uint8_t *out)
{
    uint8_t  len = 0;
    uint8_t  request = in[USB_RQ_REQ]; 
    uint8_t  value_l = in[USB_RQ_VALUE_L];
#define  ispcmd	value_l
    uint8_t  value_h = in[USB_RQ_VALUE_H];
    uint8_t  rval = 0;
    uint16_t address;


asp_debug(in[USB_RQ_TYPE]);
asp_debug(request);
    if(request == USBASP_FUNC_TRANSMIT){   /* emulate parts of ISP protocol */
//asp_debug(ispcmd);
        address = (uint16_t)(value_h <<8) | in[USB_RQ_INDEX_L];
        if(ispcmd == ISPCMD_READ_SIGNATURE){
            rval = in[USB_RQ_INDEX_L] & 3;
#if HAVE_SIGNATURE_ACCESS
	    if (rval == 3) {
		    rval = 1;
	    } else {
		    rval <<= 1;
	    }
            __cli();
  #if 1
            rval = read_signature(rval);
  #else
	    SPMCSR |= (1<<SPMEN) | (1<<SIGRD);
	    rval =  __LPM(rval);
  #endif
	    __sei();
#else
            rval = __LPM(signatureBytes+rval);
#endif
#if HAVE_EEPROM_BYTE_ACCESS
        }else if(ispcmd == ISPCMD_READ_EEPROM){
            rval = eeprom_read_byte((void *)address);
        }else if(ispcmd == ISPCMD_WRITE_EEPROM){
            eeprom_write_byte((void *)address, in[USB_RQ_INDEX_H]);
	    eeprom_busy_wait();
#endif
#if HAVE_FLASH_BYTE_ACCESS
        }else if((ispcmd & 0xf7) == ISPCMD_READ_FLASH_L) {
            address <<= 1;
	    if (ispcmd & 0x08) {  /* read FLASH upper */
		    address += 1;
	    }
#if EEPROM_SAVE_SIZE
	    if (address < EEPROM_SAVE_SIZE) {
            	rval = eeprom_read_byte((void *)address+ EEPROM_SAVE_BASE);
	    } else 
#endif
	    {
            	rval = __LPM((void *)address);
	    }
#endif
#if HAVE_FUSE_ACCESS
        }else if((ispcmd & 0xf7) == ISPCMD_READ_FUSE_L) {
	    rval = 0;
	    if (ispcmd & 0x08) rval |= 1;
	    if (value_h & 0x08) rval |= 2;
            __cli();
	    rval = read_fuse(rval);
            __sei();
#endif
#if HAVE_CHIP_ERASE
        }else if(ispcmd == ISPCMD_CHIP_ERASE && value_h == 0x80){
            addr_t addr;
            for(addr = 0; addr < (APP_FLASHEND) + 1; addr += SPM_PAGESIZE) {
                /* erase page and wait */
                //DBG1(0x33, 0, 0);

#if USE_DFU_CALL
                __cli();
                dfu_page_erase(addr);
                __sei();
#else
                __cli();
                boot_page_erase(addr);
                __sei();
                boot_spm_busy_wait();
#endif
#if EEPROM_SAVE_SIZE
		if (addr == 0) fill_vector();
#endif

            }
#endif
        }else{
            /* ignore all others, return default value == 0 */
        }
        out[3] = rval;
        len = 4;
    }else if(request == USBASP_FUNC_ENABLEPROG || request == USBASP_FUNC_SETISPSCK){
        out[0] = 0;
        len = 1;
    }else if(request == USBASP_FUNC_CONNECT){
    }else if(request == USBASP_FUNC_DISCONNECT){
	busy_led_off();
#if BOOTLOADER_CAN_EXIT
        requestBootLoaderExit = 1;      /* allow proper shutdown/close of connection */
#endif
    }else if(request == USBASP_FUNC_SETLONGADDRESS){
#if (FLASHEND > USHRT_MAX)
            currentAddress.b[2] = in[USB_RQ_INDEX_L];
            currentAddress.b[3] = in[USB_RQ_INDEX_H];
#endif
    }else if(request > USBASP_FUNC_SETISPSCK) {
    }else {
        currentAddress.b[0] = value_l;
        currentAddress.b[1] = value_h;
	len = in[USB_RQ_LENGTH_L];
	if(request == USBASP_FUNC_READEEPROM || request == USBASP_FUNC_READFLASH) {
    		//for(rval = len; rval ; rval--){
    		for(rval = 0; rval < len; rval++) {
		    if (request == USBASP_FUNC_READEEPROM) {
            		*out++ = eeprom_read_byte((void *)currentAddress.w[0]);
#if EEPROM_SAVE_SIZE
		    } else if (CURRENT_ADDRESS < EEPROM_SAVE_SIZE) {
            		    *out++ = eeprom_read_byte(
					(void *)(currentAddress.w[0]+EEPROM_SAVE_BASE));
#endif
		    } else {
            		    *out++ = pgm_read_byte((void *)CURRENT_ADDRESS);
		    }
        	    CURRENT_ADDRESS++;
		}
	} else {
            /* if(in[USB_RQ_REQ] == USBASP_FUNC_WRITEFLASH) only evaluated during writeFlash anyway */
            isLastPage = in[USB_RQ_INDEX_H] & 0x02;
#if HAVE_EEPROM_PAGED_ACCESS
            currentRequest = request;
#endif
	    bytesRemaining = len;
            len = 0; // NO ERROR @ OUT
	}
    }
    return len;
#undef ispcmd
}

uint8_t asp_out(uint8_t *data, uint8_t len)
{
#if EEPROM_SAVE_SIZE
    static uint8_t save_data[EEPROM_SAVE_SIZE];
    uint8_t j;
#endif
    uint8_t isLast;
    uint8_t i;

asp_debug(len);
asp_debug(bytesRemaining);
    //DBG1(0x31, (void *)&currentAddress.l, 4);
    if(len > bytesRemaining)
        len = bytesRemaining;
    bytesRemaining -= len;
    isLast = (bytesRemaining == 0);
#if HAVE_EEPROM_PAGED_ACCESS
    if(currentRequest == USBASP_FUNC_WRITEEEPROM){
        for(i = 0; i < len; i++){
            eeprom_write_byte((void *)(currentAddress.w[0]++), *data++);
        }
	eeprom_busy_wait();
    }else{ // USBASP_FUNC_WRITEFLASH
#endif
        for(i = 0; i < len;){
#if !HAVE_CHIP_ERASE
            if((currentAddress.w[0] & (SPM_PAGESIZE - 1)) == 0){    /* if page start: erase */
                //DBG1(0x33, 0, 0);

#if USE_DFU_CALL
                __cli();
                dfu_page_erase(CURRENT_ADDRESS);   /* erase page */
                __sei();
#else
                __cli();
                boot_page_erase(CURRENT_ADDRESS);   /* erase page */
                __sei();
                boot_spm_busy_wait();               /* wait until page is erased */
#endif

            }
#endif // CHIP_ERASE
            i += 2;
            //DBG1(0x32, 0, 0);
#if EEPROM_SAVE_SIZE
	    if (CURRENT_ADDRESS < EEPROM_SAVE_SIZE ) {
		    *(uint16_t *)(save_data+CURRENT_ADDRESS) = *(uint16_t *)data;
		    *(uint16_t *)data = __LPM_word(BOOTLOADER_ADDRESS+CURRENT_ADDRESS);
	    }
#endif
#if USE_DFU_CALL
            __cli();
            dfu_page_fill(CURRENT_ADDRESS, *(uint16_t *)data);
            __sei();
#else
            __cli();
            boot_page_fill(CURRENT_ADDRESS, *(uint16_t *)data);
            __sei();
#endif
            CURRENT_ADDRESS += 2;
            data += 2;
            /* write page when we cross page boundary or we have the last partial page */
            if((currentAddress.w[0] & (SPM_PAGESIZE - 1)) == 0 
			|| (isLast && (i >= len) && isLastPage)){
                //DBG1(0x34, 0, 0);

#if USE_DFU_CALL
                __cli();
                dfu_page_write(CURRENT_ADDRESS - 2);
                __sei();
#else
                __cli();
                boot_page_write(CURRENT_ADDRESS - 2);
                __sei();
                boot_spm_busy_wait();
                __cli();
                boot_rww_enable();
                __sei();
#endif
#if EEPROM_SAVE_SIZE
                if(currentAddress.w[0] -2 < SPM_PAGESIZE) {
        	    for(j = 0; j < EEPROM_SAVE_SIZE; j++) {
            		eeprom_write_byte((void *)(uint16_t)j+EEPROM_SAVE_BASE, save_data[j]);
		    }
		}
#endif
            }
        }
        //DBG1(0x35, (void *)&currentAddress.l, 4);
#if HAVE_EEPROM_PAGED_ACCESS
    }
#endif
    return isLast?(uint8_t)255:0; 
}
/* ------------------------------------------------------------------------ */

#ifndef ZZ_USBASP
static inline void asp_init() {
	usb_vendor_setup = asp_setup;
	usb_user_out = asp_out;
	usbInit();
	__sei();
}

int main(void)
{ 
#if BOOTLOADER_CAN_EXIT
    uint8_t i = 0, j = 0;
#endif

    /* set system_clock pre scaler 1/8 => 1/1 */
    CLKPR = (1<<CLKPCE);
    CLKPR = 0;

    //bit_set(REGCR, REGDIS); // disable internal regulator

    /* initialize  */
    wdt_disable();      /* main app may have enabled watchdog */
    bootLoaderInit();
    //DBG1(0x00, 0, 0);
#if !BOOTLOADER_EMU 
    GICR = (1 << IVCE);  /* enable change of interrupt vectors */
    GICR = (1 << IVSEL); /* move interrupts to boot flash section */
#endif

    if(bootLoaderEnterCondition()){
	busy_led_on();
        asp_init();
        do{
            usbPoll();
#if BOOTLOADER_CAN_EXIT
            if(requestBootLoaderExit){
                if(--i == 0){
                    if(--j == 0)
                        break;
                }
            }
#endif
        }while(!bootLoaderExitCondition());  /* main event loop */
    }
    usbDeviceDisconnect();
    leaveBootloader();
    return 0;
}

#ifdef EP_TX
void callback_set_baud(long baud) {
	//Baud = baud;
}
#endif
#endif // ZZ_USBASP

#ifdef EP_TX
void asp_print_status() {
#ifdef ASP_LOG
	uint8_t i,j,d;
	uint16_t offset = 0;
	usbcdc_put_pstr(PSTR("asp "));
	//usbcdc_putc('0' + asp_state);
	//usbcdc_put_pstr(PSTR(" err:"));
	//usbcdc_put_hex( last_err_cmd );
	usbcdc_put_pstr(PSTR("\r\n"));
	for(i=0; i<ASP_LOG_SIZE/16; i++) {
		d = (offset >> 8) & 0xff;
		usbcdc_put_hex(d);
		d = offset & 0xff;
		usbcdc_put_hex(d);
		usbcdc_putc(':');
		for (j=0; j<16; j++) {
			usbcdc_putc(' ');
			d = asp_log[offset];
			usbcdc_put_hex(d);
			offset++;
		}
		usbcdc_put_pstr(PSTR("\r\n"));
	}
	asp_log_idx = 0;
	memset(asp_log,0,ASP_LOG_SIZE);
#endif
}
#endif
