/* Name: desc.c
 * Project: AVR USB driver for AT90USB
 * Author: Koji Suzuki
 * Creation Date: 2007-12-10
 * Tabsize: 8
 * Copyright: (c) 2006 by Recursion Co., Ltd.
 * Copyright: (c) 2007 by Koji Suzuki (suz@nmj2.sakura.ne.jp)
 * Original Project: AVR USB driver for CDC interface on Low-Speed USB
 * Original Author: Osamu Tamura
 * License: Proprietary, free under certain conditions. See Documentation.
 */

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>
#include <util/delay.h>

#include "usbconfig.h"
#include "desc.h"

#ifdef USE_IAD
#define IAD_LEN		8
#else
#define IAD_LEN		0
#endif

/* -------------------------- Device Description --------------------------- */
#ifdef ZZ_USBASP
#define USB_CFG_EXTERNAL_CONFIG_DESCRIPTOR_LENGH    67+IAD_LEN+9

#else
#if defined(EP_MS_OUT) && defined(EP_TX) // Composite Device
#define USB_CFG_EXTERNAL_CONFIG_DESCRIPTOR_LENGH    (67+IAD_LEN+9+7+7)
#elif defined(EP_TX)
#define USB_CFG_EXTERNAL_CONFIG_DESCRIPTOR_LENGH    67+IAD_LEN
#elif defined(EP_MS_OUT)
#define USB_CFG_EXTERNAL_CONFIG_DESCRIPTOR_LENGH    (9+9+7+7)
#else
#assert ""
#endif
#endif

/* Define this to the size of your configuration descriptor if you provide
 * your own configuration descriptor at the global program memory address
 * 'usbDescrConfig'.
 */

#define  USB_CFG_VENDOR_ID  0xc0, 0x16  /* 5824 in dec, stands for VOTI */
/* USB vendor ID for the device, low byte first. If you have registered your
 * own Vendor ID, define it here. Otherwise you use obdev's free shared
 * VID/PID pair. Be sure to read USBID-License.txt for rules!
 */
#define USB_CFG_DEVICE_ID   0xdc, 0x05  /* 1500 in dec, obdev's free PID */
/* This is the ID of the product, low byte first. It is interpreted in the
 * scope of the vendor ID. If you have registered your own VID with usb.org
 * or if you have licensed a PID from somebody else, define it here. Otherwise
 * you use obdev's free shared VID/PID pair. Be sure to read the rules in
 * USBID-License.txt!
 */
#define  USB_CFG_DEVICE_VERSION  0x02, 0x01

#define  USB_CFG_IS_SELF_POWERED 0

#define USB_CFG_HID_REPORT_DESCRIPTOR_LENGTH    0

PROGMEM char usbDescrDevice[] = {    /* USB device descriptor */
    18,         /* sizeof(usbDescrDevice): length of descriptor */
    USB_DESCR_DEVICE,    /* descriptor type */
    0x01, 0x01, /* USB version supported */
#ifdef EP_TX
#ifdef USE_IAD
    0xef,	/* device class : Composite Device Class */
    2,		/* subclass */
    1,          /* protocol */
#else
    2,		/* device class */
    0,		/* subclass */
    0,          /* protocol */
#endif // USE_IAD
#else
    0,		/* device class */
    0,		/* subclass */
    0,          /* protocol */
#endif
    EP_SIZE,       /* max packet size */
    USB_CFG_VENDOR_ID,  /* 2 bytes */
    USB_CFG_DEVICE_ID,  /* 2 bytes */
    USB_CFG_DEVICE_VERSION, /* 2 bytes */
    1,          /* manufacturer string index */
    2,          /* product string index */
    0,          /* serial number string index */
    1,          /* number of configurations */
};

PROGMEM char usbDescrConfig[] = {    /* USB configuration descriptor */
    9,          /* sizeof(usbDescrConfig): length of descriptor */
    USB_DESCR_CONFIG,    /* descriptor type */
    USB_CFG_EXTERNAL_CONFIG_DESCRIPTOR_LENGH,
    0,          /* total length of data returned (including inlined descriptors) */
#ifdef ZZ_USBASP
    3,          /* number of interfaces in this configuration */
#else
#if defined(EP_TX) && defined(EP_MS_OUT)
    3,          /* number of interfaces in this configuration */
#elif defined(EP_TX)
    2,          /* number of interfaces in this configuration */
#else
    1,
#endif
#endif
    1,          /* index of this configuration */
    0,          /* configuration name string index */
#if USB_CFG_IS_SELF_POWERED
    USB_ATTR_SELFPOWER,  /* attributes */
#else
    USB_ATTR_BUSPOWER,   /* attributes */
#endif
    100/2,            /* max USB current in 2mA units */

#ifdef EP_TX
#ifdef USE_IAD
    /* interface association descriptor (IAD) : */
    8,
    USB_DESCR_IAD,
    0,		/* Associated Interface # */
    2,		/* # of Associated Interfaces */
    USB_CLASS_COMM, /* class: */
    2, /* subclass: ACM (Abstract Control Modem) */
    1, /* protocol: AT-Commands */
    0, /* string index for composite interface */
#endif
    /* interface descriptor : */
    9,          /* sizeof(usbDescrInterface): length of descriptor */
    USB_DESCR_INTERFACE, /* descriptor type */
    0,          /* index of this interface */
    0,          /* alternate setting for this interface */
    1,   /* endpoints excl 0: number of endpoint descriptors to follow */
    USB_CLASS_COMM, /* CDC Class */
    2, /* ACM (Abstract Control Modem) */
    1, /* AT-Commands */
    0,          /* string index for interface */

    /* CDC Class-Specific descriptor */
    5,           /* sizeof(usbDescrCDC_HeaderFn): length of descriptor */
    USB_DESCR_CS_INTERFACE,        /* descriptor type */
    0,           /* header functional descriptor */
    0x10, 0x01,

    4,           /* sizeof(usbDescrCDC_AcmFn): length of descriptor */
    USB_DESCR_CS_INTERFACE,        /* descriptor type */
    2,           /* abstract control management functional descriptor */
    0x02,        /* SET_LINE_CODING,    GET_LINE_CODING, SET_CONTROL_LINE_STATE    */

    5,           /* sizeof(usbDescrCDC_UnionFn): length of descriptor */
    USB_DESCR_CS_INTERFACE,        /* descriptor type */
    6,           /* union functional descriptor */
    0,           /* CDC_COMM_INTF_ID */
    1,           /* CDC_DATA_INTF_ID */

    5,           /* sizeof(usbDescrCDC_CallMgtFn): length of descriptor */
    USB_DESCR_CS_INTERFACE,        /* descriptor type */
    1,           /* call management functional descriptor */
    3,           /* allow management on data interface, handles call management by itself */
    1,           /* CDC_DATA_INTF_ID */

    /* Endpoint Descriptor */
    7,           /* sizeof(usbDescrEndpoint) */
    USB_DESCR_ENDPOINT,  /* descriptor type = endpoint */
    0x80 | EP_INT,      /* IN endpoint number 3 */
    0x03,        /* attrib: Interrupt endpoint */
    EP_INT_SIZE, 0,        /* maximum packet size */
    100,         /* in ms */

    /* Interface Descriptor  */
    9,           /* sizeof(usbDescrInterface): length of descriptor */
    USB_DESCR_INTERFACE,           /* descriptor type */
    1,           /* index of this interface */
    0,           /* alternate setting for this interface */
    2,           /* endpoints excl 0: number of endpoint descriptors to follow */
    USB_CLASS_COMM_DATA,        /* Data Interface Class Codes */
    0,
    0,           /* Data Interface Class Protocol Codes */
    0,           /* string index for interface */

    /* Endpoint Descriptor */
    7,           /* sizeof(usbDescrEndpoint) */
    USB_DESCR_ENDPOINT,  /* descriptor type = endpoint */
    0x80 | EP_RX,        /* IN endpoint number 1 */
    0x02,        /* attrib: Bulk endpoint */
    EP_RX_SIZE, 0,        /* maximum packet size */
    0,           /* in ms */

    /* Endpoint Descriptor */
    7,           /* sizeof(usbDescrEndpoint) */
    USB_DESCR_ENDPOINT,  /* descriptor type = endpoint */
    EP_TX,       /* OUT endpoint number 2 */
    0x02,        /* attrib: Bulk endpoint */
    EP_TX_SIZE, 0,        /* maximum packet size */
    0,           /* in ms */
#endif // EP_TX
#ifdef EP_MS_OUT
    /* Interface Descriptor  */
    9,           /* sizeof(usbDescrInterface): length of descriptor */
    USB_DESCR_INTERFACE,           /* descriptor type */
    2,           /* index of this interface */
    0,           /* alternate setting for this interface */
    2,           /* endpoints excl 0: number of endpoint descriptors to follow */
    USB_CLASS_MASS_STORAGE,        /* Class: Mass Storage */
    6,		 /* SubClass: SCSI transparent Command Set */
    0x50,        /* Protocol: Bulk-Only Transport */
    0,           /* string index for interface */

    /* Endpoint Descriptor */
    7,           /* sizeof(usbDescrEndpoint) */
    USB_DESCR_ENDPOINT,  /* descriptor type = endpoint */
    0x80 | EP_MS_IN,        /* IN endpoint number 1 */
    0x02,        /* attrib: Bulk endpoint */
    EP_MS_IN_SIZE, 0,        /* maximum packet size */
    0,           /* in ms */

    /* Endpoint Descriptor */
    7,           /* sizeof(usbDescrEndpoint) */
    USB_DESCR_ENDPOINT,  /* descriptor type = endpoint */
    EP_MS_OUT,       /* OUT endpoint number 2 */
    0x02,        /* attrib: Bulk endpoint */
    EP_MS_OUT_SIZE, 0,        /* maximum packet size */
    0,           /* in ms */
#endif
#ifdef ZZ_USBASP
    /* interface descriptor : */
    9,          /* sizeof(usbDescrInterface): length of descriptor */
    USB_DESCR_INTERFACE, /* descriptor type */
    2,          /* index of this interface */
    0,          /* alternate setting for this interface */
    0,   /* endpoints excl 0: number of endpoint descriptors to follow */
    0xff, /* Class */
    0, /* subclass: */
    0, /* protocol */
    0,          /* string index for interface */
#endif
};

PROGMEM int16_t  usbDescrString0[] = {
	4 | (3<<8),	/* size, descriptor type */
	0x0409		/* language index (0x0409 = US-English) */
};

PROGMEM int16_t  usbDescrString1[] = {
    14*2 | (3<<8),
    'w', 'w', 'w', '.', 'f', 'i', 's', 'c', 'h', 'l', '.', 'd', 'e', 0, 
};

PROGMEM int16_t  usbDescrString2[] = {
    7*2 | (3<<8),
    'U', 'S', 'B', 'a', 's', 'p', 0, 
};

PROGMEM int16_t  usbDescrString3[] = {
	0 | (3<<8),
};

PROGMEM int16_t  *usbDescrStringTable[4] = {
	usbDescrString0,
	usbDescrString1,
	usbDescrString2,
	usbDescrString3 
};
